const insertElementAfter = (rNode, nNode) => {
  rNode.parentNode.insertBefore(nNode, rNode.nextSibling);
};

const insertElementBefore = (rNode, nNode) => {
  rNode.parentNode.insertBefore(nNode, rNode);
};

const getNovaToken = async () => {
  const data = await browser.storage.local.get("nova_auth_token");
  return data["nova_auth_token"] || "";
};
const cleanEscapedSvg = (escapedSvg) => {
  let cleanedSvg = escapedSvg.replace(/\\"/g, '"');
  cleanedSvg = cleanedSvg.replace(/\\n/g, "");
  return cleanedSvg;
};
const saveFlyingModalPosition = (left, top) => {
  browser.storage.local.set(
    {
      flying_modal_position: { left, top },
    },
    () => {},
  );
};
const addressPurifier = (address) => {
  const cleanedAddress = address.replace(/Share$/, "").trim();

  const match = cleanedAddress.match(/^[^:]*:(.*)/);
  return match ? match[1].trim() : cleanedAddress;
};

const checkDuplicateTransaction = async (payload) => {
  return new Promise((resolve, reject) => {
    chrome.storage.local.get("current_buy_tx", (storedData) => {
      const currentTx = storedData.current_buy_tx
        ? JSON.parse(storedData.current_buy_tx)
        : null;

      if (!currentTx) {
        resolve(false);
        return;
      }

      const isDuplicate =
        currentTx.mint === payload.mint &&
        currentTx.method === payload.method &&
        currentTx.amount === payload.amount &&
        currentTx.timestamp === payload.timestamp;

      if (isDuplicate) {
        setTimeout(() => {
          chrome.storage.local
            .set({
              current_buy_tx: JSON.stringify({}),
            })
            .then(() => {});
        }, 700);
        reject(new Error("Duplicate transaction detected"));
      } else {
        resolve(false);
      }
    });
  });
};

const transactToken = async (
  mintAddress,
  method,
  value,
  authToken,
  activePresetValues,
  solAmount,
) => {
  try {
    const payload = {
      mint: mintAddress,
      method: method,
      amount: parseFloat(value) ?? null,
      buyFee: null,
      buyTip: null,
      buySlippage: null,
      sellFee: null,
      sellTip: null,
      sellSlippage: null,
      solAmount: solAmount || null,
    };

    const timestamp = Date.now() / 1000;
    const payloadWithTimestamp = {
      ...payload,
      timestamp,
    };

    await checkDuplicateTransaction(payloadWithTimestamp);

    chrome.storage.local.set({
      current_buy_tx: JSON.stringify(payloadWithTimestamp),
    });

    if (method === "buy") {
      payload.buyFee = activePresetValues["buy-fee"];
      payload.buyTip = activePresetValues["buy-tip"];
      payload.buySlippage = activePresetValues["buy-slippage"];
    } else if (method === "sell") {
      payload.sellFee = activePresetValues["sell-fee"];
      payload.sellTip = activePresetValues["sell-tip"];
      payload.sellSlippage = activePresetValues["sell-slippage"];
    } else if (method === "snipe") {
    }

    const response = await fetch("https://api.tradeonnova.io/api-v1/transact", {
      method: "POST",
      headers: {
        "Content-Type": "application/json",
        "X-Auth-Token": authToken,
      },
      body: JSON.stringify(payload),
    });

    if (!response.ok) {
      throw new Error(`Failed to buy token:, ${response.statusText}`);
    }

    const result = await response.json();

    if (result.success) {
    } else {
    }

    return result.success;
  } catch (error) {}
};
